@@ -51,6 +51,9 @@ static R_CallMethodDef CallEntries[] = {
     CALLDEF(R_mpfr_set_debug, 1),
     CALLDEF(R_mpfr_set_default_prec, 1),
     CALLDEF(R_mpfr_get_default_prec, 0),
+    CALLDEF(R_mpfr_get_erange, 1),
+    CALLDEF(R_mpfr_set_erange, 2),
+    CALLDEF(R_mpfr_prec_range, 1),
     CALLDEF(R_mpfr_get_version, 0),
 
     CALLDEF(const_asMpfr, 2),
