@@ -242,7 +242,7 @@ int my_mpfr_choose (mpfr_t R, long n, mpfr_t X, mpfr_rnd_t RND)
     int ans;
     long i;
     mpfr_t r, x;
-    mp_prec_t p_X = mpfr_get_prec(X);
+    mpfr_prec_t p_X = mpfr_get_prec(X);
 
     mpfr_init2(x, p_X); mpfr_set(x, X, RND);
     mpfr_init2(r, p_X);
@@ -275,7 +275,7 @@ int my_mpfr_poch (mpfr_t R, long n, mpfr_t X, mpfr_rnd_t RND)
     int ans;
     long i;
     mpfr_t r, x;
-    mp_prec_t p_X = mpfr_get_prec(X);
+    mpfr_prec_t p_X = mpfr_get_prec(X);
 
     mpfr_init2(x, p_X); mpfr_set(x, X, RND);
     mpfr_init2(r, p_X);
@@ -308,7 +308,7 @@ int my_mpfr_round (mpfr_t R, long prec, mpfr_t X, mpfr_rnd_t RND)
     if(prec > MPFR_PREC_MAX)
 	error("prec = %d > %d  is too large", prec, MPFR_PREC_MAX);
     mpfr_set(R, X, RND);
-    ans = mpfr_prec_round(R, (mp_prec_t) prec, RND);
+    ans = mpfr_prec_round(R, (mpfr_prec_t) prec, RND);
     return ans;
 }
 
@@ -318,13 +318,17 @@ SEXP R_mpfr_get_version(void) {
     return mkString(mpfr_get_version());
 }
 
-SEXP R_mpfr_set_debug(SEXP I) {
-    /* Set or get the C-global debugging level : */
+/* Set or get the C-global debugging level --
+ * only used in R_mpfr_dbg_printf() --> ./Rmpfr_utils.h
+*/
+SEXP R_mpfr_set_debug(SEXP I)
+{
     if(LENGTH(I) < 1 || INTEGER(I)[0] == NA_INTEGER)
 	return ScalarInteger(R_mpfr_debug_);
     /* else : */
+    int prev = R_mpfr_debug_;
     R_mpfr_debug_ = asInteger(I);
-    return I;
+    return ScalarInteger(prev);
 }
 
 SEXP R_mpfr_get_default_prec(void) {
@@ -332,9 +336,68 @@ SEXP R_mpfr_get_default_prec(void) {
 }
 
 SEXP R_mpfr_set_default_prec(SEXP prec) {
-    SEXP ans = ScalarInteger((int) mpfr_get_default_prec());
-    mpfr_set_default_prec((mp_prec_t) asInteger(prec));
-    return ans;
+    // return the previous value
+    int prev = (int) mpfr_get_default_prec();
+    mpfr_set_default_prec((mpfr_prec_t) asInteger(prec));
+    return ScalarInteger(prev);
+}
+
+typedef enum { E_min = 1, E_max,
+	       min_emin, max_emin, min_emax, max_emax } erange_kind;
+
+SEXP R_mpfr_get_erange(SEXP kind_) {
+    erange_kind kind = asInteger(kind_);
+/* MUST be sync'ed with  ../R/mpfr.R
+ *                       ~~~~~~~~~~~ where  .Summary.codes <-
+
+ */
+    mpfr_exp_t r;
+    switch(kind) {
+    case E_min:    r = mpfr_get_emin();     break;
+    case E_max:    r = mpfr_get_emax();     break;
+    case min_emin: r = mpfr_get_emin_min(); break;
+    case max_emin: r = mpfr_get_emin_max(); break;
+    case min_emax: r = mpfr_get_emax_min(); break;
+    case max_emax: r = mpfr_get_emax_max(); break;
+    default:
+	error("invalid kind (code = %d) in R_mpfr_get_erange()", kind);
+    }
+    R_mpfr_dbg_printf(1,"R_mpfr_get_erange(%d): %ld\n", kind, (long)r);
+    return (kind <= E_max) ? ScalarInteger((int) r) : ScalarReal((double) r);
+}
+
+SEXP R_mpfr_set_erange(SEXP kind_, SEXP val) {
+    erange_kind kind = asInteger(kind_);
+    mpfr_exp_t exp_val;
+    if(isInteger(val))
+	exp_val = asInteger(val);// assume this is always valid to set
+
+    else { // we allow larger values from the R side
+	PROTECT(val = coerceVector(val, REALSXP));
+	exp_val = (mpfr_exp_t) asReal(val);
+	UNPROTECT(1);
+    }
+
+    int i_err;
+    switch(kind) {
+    case E_min: i_err = mpfr_set_emin(exp_val); break;
+    case E_max: i_err = mpfr_set_emax(exp_val); break;
+    default:
+	error("invalid kind (code = %d) in R_mpfr_set_erange()", kind);
+    }
+    if(i_err) warning("e%s exponent could not be set to %ld (code %d)",
+		      (kind == E_min) ? "min" : "max", (long)exp_val, i_err);
+    return ScalarInteger(i_err);
+}
+
+SEXP R_mpfr_prec_range(SEXP ind) {
+    long r = (long) (
+	(INTEGER(ind)[0] == 1)
+	? MPFR_PREC_MIN
+	: MPFR_PREC_MAX);
+    R_mpfr_dbg_printf(1,"R_mpfr_prec_range(): %ld\n", r);
+    // in 64 bit, int << long, so go "2nd best":
+    return ScalarReal((double)r);
 }
 
 
@@ -355,7 +418,9 @@ SEXP const_asMpfr(SEXP I, SEXP prec)
 {
     SEXP val;
     mpfr_t r;
-    mpfr_init2(r, asInteger(prec));
+    int i_p = asInteger(prec);
+    R_mpfr_check_prec(i_p);
+    mpfr_init2(r, i_p);
 
     switch(asInteger(I)) {
     case 1: mpfr_const_pi     (r, MPFR_RNDN); break;
@@ -406,7 +471,9 @@ SEXP R_mpfr_fac (SEXP n_, SEXP prec)
     } else {
 	nn = INTEGER(n_);
     }
-    mpfr_init2(r_i, (mp_prec_t) asInteger(prec));
+    int i_p = asInteger(prec);
+    R_mpfr_check_prec(i_p);
+    mpfr_init2(r_i, i_p);
     for(i=0; i < n; i++) {
 	// never happens when called from R:
 	if(nn[i] < 0) error("R_mpfr_fac(%d): negative n.", nn[i]);
